angular.module('beamng.apps')

.constant('AI_Options', {

    aiMode: [
      { txt: 'Выключено', val: 'disabled' },
      { txt: 'Траффик',   val: 'traffic' },
      { txt: 'Случайно',   val: 'random'   },
      { txt: 'Охватывающе',     val: 'span'     },
      { txt: 'Ручной',   val: 'manual'   },
      { txt: 'Погоня',    val: 'chase'    },
      { txt: 'Следовать',    val: 'follow'  },
      { txt: 'Бежать',     val: 'flee'     },
      { txt: 'Остановка', val: 'stop'     }
    ],

    vehicleDebugMode: [
      { txt: 'Выключено',    val: 'off'        },
      { txt: 'Цель',      val: 'target'     },
      { txt: 'Скорость',      val: 'speeds'     },
      { txt: 'Траектория',  val: 'trajectory' },
      { txt: 'Маршрут',       val: 'route'      }
    ],

    speedMode: [
      { txt: 'Выкл',   val: 'off'    },
      { txt: 'Удержание',   val: 'set'    },
      { txt: 'Законный', val: 'legal'  },
      { txt: 'Лимит', val: 'limit'  }
    ],

    driveInLaneFlag: [
      { txt: 'Выкл',   val: 'off'    },
      { txt: 'Вкл',    val: 'on'     }
    ],

    avoidCarsMaster: [
      { txt: 'Автоматически',  val: 'auto'   },
      { txt: 'Выкл',   val: 'off'    },
      { txt: 'Вкл',    val: 'on'     }
    ]
})


.directive('aiControl', ['AI_Options', 'bngApi', function (AI_Options, bngApi) {
  return {
    templateUrl: 'modules/apps/AIControl/app.html',
    replace: false,
    restrict: 'E',
    scope: false,
    link: function (scope, element, attrs) {
      var ctrl = scope.ai;
      ctrl.populate();

      scope.$on('AIStateChange', function (_, data) {
        scope.$evalAsync(function () {
          bngApi.engineLua('extensions.hook("trackAISingleVeh", ("' + data.mode + '"))');
          ctrl.mode.value = data.mode;
          if (data.mode.value != 'manual') ctrl.mapNodes.value = null;

          ctrl.aggression.value = data.extAggression;
          ctrl.driveInLaneFlag.value = data.driveInLaneFlag;
          ctrl.avoidCarsMaster.value = data.avoidCarsMaster;
          ctrl.vehicleDebugMode.value = data.debugMode;
          ctrl.targetObjects.value = data.targetObjectID;
          ctrl.mapNodes.value = data.manualTargetName;
          ctrl.routeSpeed.value = Math.round(data.routeSpeed * ctrl.distanceUnits.mult);
          ctrl.speedMode.value = data.speedMode;
          ctrl.drivability.value = data.cutOffDrivability;
        });
      });

      bngApi.engineLua('settings.requestState()');

      scope.$on('VehicleReset', function (_, data) {
        ctrl._updateMapState();
      });

      scope.$on('VehicleFocusChanged', function (_, data) {
        ctrl._updateMapState();
      });
    },

    controller: function ($scope, $element, $attrs) {
      var vm = this;

      var units = {
        'metric': {label: 'km/h', mult: 3.6},
        'imperial': {label: 'mph', mult: 2.23694}
      };

      vm.distanceUnits = {};

      $scope.$on('SettingsChanged', function (event, data) {
        $scope.$evalAsync(function () {
          vm.distanceUnits = units[data.values.uiUnitLength];
        });
      });

      vm.mode = {value: null, options: AI_Options.aiMode};
      vm.aggression = {value: 1};
      vm.driveInLaneFlag = {value: null, options: AI_Options.driveInLaneFlag};
      vm.avoidCarsMaster = {value: null, options: AI_Options.avoidCarsMaster};
      vm.routeSpeed = {value: null};
      vm.speedMode = {value: null, options: AI_Options.speedMode};
      vm.vehicleDebugMode = {value: null, options: AI_Options.vehicleDebugMode};
      vm.mapNodes = {value: null, options: []};
      vm.targetObjects = {value: null, options: []};
      vm.drivability = {value: 0};

      vm.changeMode = function () {
        var cmd = `ai.setState( ${bngApi.serializeToLua({mode: vm.mode.value})} )`;
        bngApi.activeObjectLua(cmd);
      };

      vm.changeAggression = function () {
        var cmd = `ai.setAggression(${vm.aggression.value})`;
        bngApi.activeObjectLua(cmd);
      };

      vm.changeCutOffDrivability = function () {
        var cmd = `ai.setCutOffDrivability(${vm.drivability.value})`;
        bngApi.activeObjectLua(cmd);
        console.log('Changed cut off drivability')
      }

      vm.changeLaneDriving = function () {
        var cmd = `ai.driveInLane("${vm.driveInLaneFlag.value}")`;
        bngApi.activeObjectLua(cmd);
      };

      vm.changeTrafficAwareness = function () {
        var cmd = `ai.setAvoidCars("${vm.avoidCarsMaster.value}")`;
        bngApi.activeObjectLua(cmd);
      };

      vm.changeSpeed = function () {
        var siSpeed = vm.routeSpeed.value / vm.distanceUnits.mult;
        var cmd = `ai.setSpeed(${siSpeed})`;
        bngApi.activeObjectLua(cmd);
      };

      vm.changeSpeedMode = function () {
        var cmd = `ai.setSpeedMode("${vm.speedMode.value}")`;
        bngApi.activeObjectLua(cmd);
      };

      vm.changeVehicleDebugMode = function () {
        var cmd = `ai.setVehicleDebugMode( ${bngApi.serializeToLua({debugMode: vm.vehicleDebugMode.value})} )`;
        bngApi.activeObjectLua(cmd);
      };

      vm.changeMapNode = function () {

        var cmd = `ai.setTarget("${vm.mapNodes.value}")`
        bngApi.activeObjectLua(cmd);
      };

      vm.changeTargetObject = function () {
        var cmd = `ai.setTargetObjectID(${vm.targetObjects.value})`;
        bngApi.activeObjectLua(cmd);
      };

      vm._updateAiState = function () {
        bngApi.activeObjectLua('ai.getState()', (response) => {
          $scope.$evalAsync(function () {
            vm.mode.value = response.mode;

            if (vm.mode.value != 'manual') vm.mapNodes.value = null;

            vm.aggression.value = response.extAggression;
            vm.driveInLaneFlag.value = response.driveInLaneFlag;
            vm.avoidCarsMaster.value = response.avoidCarsMaster;
            vm.vehicleDebugMode.value = response.debugMode;
            vm.targetObjects.value = response.targetObjectID;
            vm.mapNodes.value = response.manualTargetName;
            vm.routeSpeed.value = response.routeSpeed * vm.distanceUnits.mult;
            vm.speedMode.value = response.speedMode;
            vm.drivability.value = response.drivability;
          });
        });
      };

      vm._updateMapState = function () {
        bngApi.engineLua('map.getState()', (response) => {

          $scope.$evalAsync(function () {
            vm.targetObjects.options =  Object.keys(response.objects)
              .filter(x => !response.objects[x].active)
              .map(x => ({
                id: response.objects[x].id,
                txt: `${response.objects[x].name}  (${response.objects[x].licensePlate}, ${x})`
              }));
          });
        });
      };

      vm._updateNodes = function () {
        bngApi.engineLua('map.getMap()', (response) => {
          $scope.$evalAsync(function () {
            vm.mapNodes.options = Object.keys(response.nodes).sort();
          });
        });
      };

      vm.populate = function () {
        vm._updateAiState();
        vm._updateMapState();
        vm._updateNodes();
      };

    },
    controllerAs: 'ai'
  };
}]);