(function () {
  'use strict';
  angular.module('beamng.apps')
  .directive('damageApp', ['bngApi', 'StreamsManager', '$sce', '$timeout', function (bngApi, StreamsManager, $sce, $timeout) {
    return {
      template:
      `
        <div>
          <div style="width: 100%; height: 100%" ng-include="'modules/apps/DamageAppVehicleSimple/damage_car.svg'" onload="svgLoaded()"></div>
        </div>
      `,
      replace: true,
      link: function ($scope, element, attrs) {
        // Streams:
        var streamsList = ['wheelThermalData', 'engineInfo'];
        StreamsManager.add(streamsList);

        var greenColor  = `rgba(0,   255, 0, 0.6)`,
            orangeColor = `rgba(255, 132, 0, 0.6)`,
            redColor    = `rgba(255, 0,   0, 0.6)`,
            noDataColor = 'rgba(0,   0,   0, 0  )',
            damageQueue = [],     
            hasDamage = 0, 
            textDisplayTime = 2000,
            beams = {},
            appDisplayed = 0,
            animTimeout,
            damageTimeout,
            textFunction,
            permanentDamage = 0;

        $scope.svgLoaded = function () {
          var svg = element[0].children[0].children[0],
              svgCarGroup = hu('#carGroup', svg),
              svgDamageTextContainer = hu('#dmgContainer', svg),
              svgDamageText = hu('#dmgText', svg);

          var componentDamageMap =  {
            body : {
              FL:                   { svgId: '#bodyFL',       priority: 2, damageDisplayed: 0      },
              FR:                   { svgId: '#bodyFR',       priority: 2, damageDisplayed: 0      },
              ML:                   { svgId: '#bodyML',       priority: 2, damageDisplayed: 0      },
              MR:                   { svgId: '#bodyMR',       priority: 2, damageDisplayed: 0      },
              RL:                   { svgId: '#bodyRL',       priority: 2, damageDisplayed: 0      },
              RR:                   { svgId: '#bodyRR',       priority: 2, damageDisplayed: 0      }
            },

            engine: {
              coolantHot:           { svgId: '#engine',       priority: 0, damageDisplayed: 0, damageText: 'Перегрев охлаждающей жидкости'                      },
              oilStarvation:        { svgId: '#engine',       priority: 0, damageDisplayed: 0, damageText: 'Масляное голодание'                           },
              oilHot:               { svgId: '#engine',       priority: 0, damageDisplayed: 0, damageText: 'Перегрев масла'                          },
              pistonRingsDamaged:   { svgId: '#engine',       priority: 0, damageDisplayed: 0, damageText: 'Поршневые кольца повреждены'                     },
              rodBearingsDamaged:   { svgId: '#engine',       priority: 0, damageDisplayed: 0, damageText: 'Поврежденные стержневые подшипники'                     },
              headGasketDamaged:    { svgId: '#engine',       priority: 0, damageDisplayed: 0, damageText: 'Прокладка головки повреждена'                      },
              turbochargerHot:      { svgId: '#engine',       priority: 0, damageDisplayed: 0, damageText: 'Перегрев турбокомпрессора'                 },
              engineIsHydrolocking: { svgId: '#engine',       priority: 0, damageDisplayed: 0, damageText: 'Двигатель гидроблокирован'                   },
              engineReducedTorque:  { svgId: '#engine',       priority: 0, damageDisplayed: 0, damageText: 'Снижение крутящего момента двигателя'                    },
              mildOverrevDamage:    { svgId: '#engine',       priority: 0, damageDisplayed: 0, damageText: 'Умеренное превышение оборотов'                     },
              overRevDanger:        { svgId: '#engine',       priority: 0, damageDisplayed: 0, damageText: 'Риск превышения оборотов',                           },
              overTorqueDanger:     { svgId: '#engine',       priority: 0, damageDisplayed: 0, damageText: 'Риск превышения крутящего момента'                         },
              engineHydrolocked:    { svgId: '#engine',       priority: 1, damageDisplayed: 0, damageText: 'Двигатель гидроблокирован'                    },
              engineDisabled:       { svgId: '#engine',       priority: 1, damageDisplayed: 0, damageText: 'Двигатель отключен'                          },
              blockMelted:          { svgId: '#engine',       priority: 1, damageDisplayed: 0, damageText: 'Блок расплавился'                             },
              engineLockedUp:       { svgId: '#engine',       priority: 1, damageDisplayed: 0, damageText: 'Двигатель заблокирован'                         },
              radiatorLeak:         { svgId: '#radiator',     priority: 1, damageDisplayed: 0, damageText: 'Утечка радиатора'                         }
            },
            powertrain: {
              wheelaxleFL:          { svgId: '#wheelaxleFL',  priority: 1, damageDisplayed: 0, damageText: 'Сломана Передняя Левая Ось'                   },
              wheelaxleFR:          { svgId: '#wheelaxleFR',  priority: 1, damageDisplayed: 0, damageText: 'Сломана Передняя Правая Ось'                  },
              wheelaxleRL:          { svgId: '#wheelaxleRL',  priority: 1, damageDisplayed: 0, damageText: 'Сломана Задняя Левая Ось'                    },
              wheelaxleRR:          { svgId: '#wheelaxleRR',  priority: 1, damageDisplayed: 0, damageText: 'СломанаЗадняя Правая Ось'                   },
              driveshaft:           { svgId: '#driveshaft' ,  priority: 1, damageDisplayed: 0, damageText: 'Карданный вал сломан'                        },
              driveshaft_F:         { svgId: '#driveshaft' ,  priority: 1, damageDisplayed: 0, damageText: 'Передний Карданный вал сломан'                  },
              mainEngine:           { svgId: '#engine'     ,  priority: 1, damageDisplayed: 0, damageText: 'Двигатель Сломан'                            }
            },
            energyStorage: {
              mainTank:             { svgId: '#fueltank',     priority: 1, damageDisplayed: 0, damageText: 'Поврежден Топливный Бак'                        }
            },
            wheels: {
              tireFR:               { svgId: '#tireFR',       priority: 0, damageDisplayed: 0, damageText: 'Лопнула Передняя Правая Шина'                   },
              tireFL:               { svgId: '#tireFL',       priority: 0, damageDisplayed: 0, damageText: 'Лопнула Передняя Левая Шина'                    },
              tireRL:               { svgId: '#tireRL',       priority: 0, damageDisplayed: 0, damageText: 'Лопнула Задняя Левая Шина'                     },
              tireRR:               { svgId: '#tireRR',       priority: 0, damageDisplayed: 0, damageText: 'Лопнула Задняя Правая Шина'                    },

              brakeFL:              { svgId: '#brakeFL',      priority: 1, damageDisplayed: 0, damageText: 'FL Тормоз Поврежден'                         },
              brakeFR:              { svgId: '#brakeFR',      priority: 1, damageDisplayed: 0, damageText: 'FR Тормоз Поврежден'                         },
              brakeRL:              { svgId: '#brakeRL',      priority: 1, damageDisplayed: 0, damageText: 'RL Тормоз Поврежден'                         },
              brakeRR:              { svgId: '#brakeRR',      priority: 1, damageDisplayed: 0, damageText: 'RR Тормоз Поврежден'                         },

              brakeOverHeatFL:      { svgId: '#brakeFL',      priority: 0, damageDisplayed: 0, damageText: 'Снижение эффективности тормоза FL',  tempDamage: 1     },
              brakeOverHeatFR:      { svgId: '#brakeFR',      priority: 0, damageDisplayed: 0, damageText: 'Снижение эффективности тормоза FR',  tempDamage: 1     },
              brakeOverHeatRL:      { svgId: '#brakeRL',      priority: 0, damageDisplayed: 0, damageText: 'Снижение эффективности тормоза RL',  tempDamage: 1     },
              brakeOverHeatRR:      { svgId: '#brakeRR',      priority: 0, damageDisplayed: 0, damageText: 'Снижение эффективности тормоза RR',  tempDamage: 1     },

              FL:                   { svgId: '#tireFL',       priority: 1, damageDisplayed: 0, damageText: 'Сломана Передняя Левая Шина'                   },
              FR:                   { svgId: '#tireFR',       priority: 1, damageDisplayed: 0, damageText: 'Сломана Передняя Правая Шина'                  },
              RL:                   { svgId: '#tireRL',       priority: 1, damageDisplayed: 0, damageText: 'Сломана Задняя Левая Шина'                    },
              RR:                   { svgId: '#tireRR',       priority: 1, damageDisplayed: 0, damageText: 'Сломана Задняя Правая Шина'                   }
            }
          }

          function showText() {
            if (damageQueue && damageQueue.length > 0) {
              svgDamageText.css({opacity:1}).text(damageQueue[0].damageText);
              svgDamageTextContainer.css({opacity:1});
              damageQueue.splice(0, 1); 
              animTimeout = $timeout(showText, textDisplayTime);
            }
            else {
              svgDamageText.css({opacity:0}).text('');
              svgDamageTextContainer.css({opacity:0});

              if (permanentDamage === 0) {
                damageTimeout = $timeout(function() {
                  svgCarGroup.animate({opacity: 0}, 200);
                }, 1000);
              }

              appDisplayed = 0;
              damageQueue = [];
              $timeout.cancel(animTimeout);
            }
          };

          function showApp(arr) {
            svgCarGroup.animate({opacity: 1,},200);
            showText();
          };

          function reset() {
            for (var key in componentDamageMap) {
              for (var val in componentDamageMap[key]) {
                hu(componentDamageMap[key][val].svgId, svg).css({
                  fill: noDataColor
                });
                componentDamageMap[key][val].damageDisplayed = 0;
                hu(componentDamageMap[key][val].svgId, svg).n.classList.remove("flashAnim");
              }
            }
            hasDamage = 0;
            permanentDamage = 0;
            appDisplayed = 0;
            damageQueue = [];
            showApp();
          };

          function setDamage(component, color, anim) {
            hu(component.svgId, svg).css({
              fill: color
            }).attr({
              class: anim
            }).on('webkitAnimationEnd', function (){
              hu(component.svgId, svg).n.classList.remove("flashAnim");
            });
          };

          function checkDamage(type, component, data) {
            if (componentDamageMap[type] && componentDamageMap[type][component] !== undefined) {
              var damagedComponent = componentDamageMap[type][component];
              if (damagedComponent.damageDisplayed === 0) {
                if (data[type][component] === true || data[type][component] > 0) {
                  if (damagedComponent.priority === 1) {
                    permanentDamage = 1;
                    setDamage(damagedComponent, redColor, 'flashAnim')
                  } else if (damagedComponent.priority === 0) {
                    permanentDamage = 1;
                    setDamage(damagedComponent, orangeColor, 'flashAnim');
                  } else if (damagedComponent.priority === 2 ) {
                    var damageAmount = Math.round(data[type][component] * 1000);
                    var bodyColor = `rgba(${150+damageAmount}, ${150-damageAmount}, 0, 0.6)`
                    setDamage(damagedComponent, bodyColor, '');
                  }
                  hasDamage = 1;
                  if (damagedComponent.damageText !== undefined && damagedComponent.damageDisplayed === 0) {
                    damageQueue.push(damagedComponent); 
                    damagedComponent.damageDisplayed = 1;
                  }
                }
              } else if (damagedComponent.tempDamage) {
                if (data[type][component] === true || data[type][component] > 0) {
                  setDamage(damagedComponent, orangeColor, 'flashAnim');
                }
                else if (data[type][component] === false || data[type][component] === 0) {
                  setDamage(damagedComponent, noDataColor, 'flashAnim');
                }
              }
            }
          };

          $scope.$on('DamageData', (ev, data) => {
            for (var key in data) {
              for (var val in data[key]) {
                checkDamage(key, val, data);
              }
            }
            if (appDisplayed === 0 && hasDamage) {
              appDisplayed = 1;
              showApp(damageQueue);
            }
          });

          $scope.$on('DamageMessage', (ev, data) => {
            damageQueue.push(data);
            showApp();
          });

          $scope.$on('VehicleReset', function() {
            reset();
          });

          $scope.$on('VehicleChange', function() {
            reset();
          });

          $scope.$on('VehicleFocusChanged', function(evt, data) {
            if (data.mode === true) {
              reset();
            }
          });

          $scope.$on('$destroy', function () {
            StreamsManager.remove(streamsList);
          });
        }
      }
    };
  }]);
})();
